@@ -538,6 +538,9 @@ void mGUIRun(struct mGUIRunner* runner, const char* path) {
 				++frame;
 			}
 		}
+		if (!running) {
+			break;
+		}
 
 		if (runner->paused) {
 			runner->paused(runner);
@@ -648,7 +651,7 @@ void mGUIRunloop(struct mGUIRunner* runner) {
 			mInputMapLoad(&runner->params.keyMap, runner->keySources[i].id, mCoreConfigGetInput(&runner->config));
 		}
 	}
-	while (true) {
+	while (!runner->running || runner->running(runner)) {
 		char path[PATH_MAX];
 		const char* preselect = mCoreConfigGetValue(&runner->config, "lastGame");
 		if (preselect) {
