@@ -425,7 +425,7 @@ void _GBMBC1(struct GB* gb, uint16_t address, uint8_t value) {
 	int stride = 1 << memory->mbcState.mbc1.multicartStride;
 	switch (address >> 13) {
 	case 0x0:
-		switch (value) {
+		switch (value & 0xF) {
 		case 0:
 			memory->sramAccess = false;
 			break;
@@ -474,9 +474,9 @@ void _GBMBC2(struct GB* gb, uint16_t address, uint8_t value) {
 	struct GBMemory* memory = &gb->memory;
 	int shift = (address & 1) * 4;
 	int bank = value & 0xF;
-	switch (address >> 13) {
+	switch ((address & 0xC100) >> 8) {
 	case 0x0:
-		switch (value) {
+		switch (value & 0x0F) {
 		case 0:
 			memory->sramAccess = false;
 			break;
@@ -485,7 +485,7 @@ void _GBMBC2(struct GB* gb, uint16_t address, uint8_t value) {
 			break;
 		default:
 			// TODO
-			mLOG(GB_MBC, STUB, "MBC1 unknown value %02X", value);
+			mLOG(GB_MBC, STUB, "MBC2 unknown value %02X", value);
 			break;
 		}
 		break;
@@ -495,7 +495,10 @@ void _GBMBC2(struct GB* gb, uint16_t address, uint8_t value) {
 		}
 		GBMBCSwitchBank(gb, bank);
 		break;
-	case 0x5:
+	case 0x80:
+	case 0x81:
+	case 0x82:
+	case 0x83:
 		if (!memory->sramAccess) {
 			return;
 		}
@@ -511,6 +514,9 @@ void _GBMBC2(struct GB* gb, uint16_t address, uint8_t value) {
 }
 
 static uint8_t _GBMBC2Read(struct GBMemory* memory, uint16_t address) {
+	if (!memory->sramAccess) {
+		return 0xFF;
+	}
 	address &= 0x1FF;
 	int shift = (address & 1) * 4;
 	return (memory->sramBank[(address >> 1)] >> shift) | 0xF0;
@@ -518,7 +524,7 @@ static uint8_t _GBMBC2Read(struct GBMemory* memory, uint16_t address) {
 
 void _GBMBC3(struct GB* gb, uint16_t address, uint8_t value) {
 	struct GBMemory* memory = &gb->memory;
-	int bank = value & 0x7F;
+	int bank = value;
 	switch (address >> 13) {
 	case 0x0:
 		switch (value) {
@@ -536,6 +542,9 @@ void _GBMBC3(struct GB* gb, uint16_t address, uint8_t value) {
 		}
 		break;
 	case 0x1:
+		if (gb->memory.romSize < GB_SIZE_CART_BANK0 * 0x80) {
+			bank &= 0x7F;
+		}
 		if (!bank) {
 			++bank;
 		}
