@@ -132,7 +132,9 @@ void GBAUnloadROM(struct GBA* gba) {
 		if (gba->yankedRomSize) {
 			gba->yankedRomSize = 0;
 		}
+#if !defined(FIXED_ROM_BUFFER) && !defined(__wii__)
 		mappedMemoryFree(gba->memory.rom, SIZE_CART0);
+#endif
 	}
 
 	if (gba->romVf) {
@@ -433,7 +435,7 @@ bool GBALoadROM(struct GBA* gba, struct VFile* vf) {
 
 bool GBALoadSave(struct GBA* gba, struct VFile* sav) {
 	GBASavedataInit(&gba->memory.savedata, sav);
-	return true;
+	return sav;
 }
 
 void GBAYankROM(struct GBA* gba) {
@@ -444,12 +446,20 @@ void GBAYankROM(struct GBA* gba) {
 }
 
 void GBALoadBIOS(struct GBA* gba, struct VFile* vf) {
-	gba->biosVf = vf;
+	if (vf->size(vf) != SIZE_BIOS) {
+		mLOG(GBA, WARN, "Incorrect BIOS size");
+		return;
+	}
 	uint32_t* bios = vf->map(vf, SIZE_BIOS, MAP_READ);
 	if (!bios) {
 		mLOG(GBA, WARN, "Couldn't map BIOS");
 		return;
 	}
+	if (gba->biosVf) {
+		gba->biosVf->unmap(gba->biosVf, gba->memory.bios, SIZE_BIOS);
+		gba->biosVf->close(gba->biosVf);
+	}
+	gba->biosVf = vf;
 	gba->memory.bios = bios;
 	gba->memory.fullBios = 1;
 	uint32_t checksum = GBAChecksum(gba->memory.bios, SIZE_BIOS);
