@@ -353,10 +353,10 @@ static void GBASetActiveRegion(struct ARMCore* cpu, uint32_t address) {
 			case REGION_WORKING_IRAM: \
 				/* This doesn't handle prefetch clobbering */ \
 				if (cpu->gprs[ARM_PC] & 2) { \
-					value |= cpu->prefetch[0] << 16; \
-				} else { \
 					value <<= 16; \
 					value |= cpu->prefetch[0]; \
+				} else { \
+					value |= cpu->prefetch[0] << 16; \
 				} \
 				break; \
 			default: \
@@ -390,10 +390,15 @@ static void GBASetActiveRegion(struct ARMCore* cpu, uint32_t address) {
 	wait += waitstatesRegion[REGION_PALETTE_RAM];
 
 #define LOAD_VRAM \
-	if ((address & 0x0001FFFF) < SIZE_VRAM) { \
-		LOAD_32(value, address & 0x0001FFFC, gba->video.vram); \
+	if ((address & 0x0001FFFF) >= SIZE_VRAM) { \
+		if ((address & (SIZE_VRAM | 0x00014000)) == SIZE_VRAM && (GBARegisterDISPCNTGetMode(gba->memory.io[REG_DISPCNT >> 1]) >= 3)) { \
+			mLOG(GBA_MEM, GAME_ERROR, "Bad VRAM Load32: 0x%08X", address); \
+			value = 0; \
+		} else { \
+			LOAD_32(value, address & 0x00017FFC, gba->video.vram); \
+		} \
 	} else { \
-		LOAD_32(value, address & 0x00017FFC, gba->video.vram); \
+		LOAD_32(value, address & 0x0001FFFC, gba->video.vram); \
 	} \
 	wait += waitstatesRegion[REGION_VRAM];
 
@@ -520,10 +525,15 @@ uint32_t GBALoad16(struct ARMCore* cpu, uint32_t address, int* cycleCounter) {
 		LOAD_16(value, address & (SIZE_PALETTE_RAM - 2), gba->video.palette);
 		break;
 	case REGION_VRAM:
-		if ((address & 0x0001FFFF) < SIZE_VRAM) {
-			LOAD_16(value, address & 0x0001FFFE, gba->video.vram);
-		} else {
+		if ((address & 0x0001FFFF) >= SIZE_VRAM) {
+			if ((address & (SIZE_VRAM | 0x00014000)) == SIZE_VRAM && (GBARegisterDISPCNTGetMode(gba->memory.io[REG_DISPCNT >> 1]) >= 3)) {
+				mLOG(GBA_MEM, GAME_ERROR, "Bad VRAM Load16: 0x%08X", address);
+				value = 0;
+				break;
+			}
 			LOAD_16(value, address & 0x00017FFE, gba->video.vram);
+		} else {
+			LOAD_16(value, address & 0x0001FFFE, gba->video.vram);
 		}
 		break;
 	case REGION_OAM:
@@ -558,7 +568,7 @@ uint32_t GBALoad16(struct ARMCore* cpu, uint32_t address, int* cycleCounter) {
 		break;
 	case REGION_CART2_EX:
 		wait = memory->waitstatesNonseq16[address >> BASE_OFFSET];
-		if (memory->savedata.type == SAVEDATA_EEPROM) {
+		if (memory->savedata.type == SAVEDATA_EEPROM || memory->savedata.type == SAVEDATA_EEPROM512) {
 			value = GBASavedataReadEEPROM(&memory->savedata);
 		} else if ((address & (SIZE_CART0 - 1)) < memory->romSize) {
 			LOAD_16(value, address & (SIZE_CART0 - 2), memory->rom);
@@ -631,10 +641,15 @@ uint32_t GBALoad8(struct ARMCore* cpu, uint32_t address, int* cycleCounter) {
 		value = ((uint8_t*) gba->video.palette)[address & (SIZE_PALETTE_RAM - 1)];
 		break;
 	case REGION_VRAM:
-		if ((address & 0x0001FFFF) < SIZE_VRAM) {
-			value = ((uint8_t*) gba->video.vram)[address & 0x0001FFFF];
-		} else {
+		if ((address & 0x0001FFFF) >= SIZE_VRAM) {
+			if ((address & (SIZE_VRAM | 0x00014000)) == SIZE_VRAM && (GBARegisterDISPCNTGetMode(gba->memory.io[REG_DISPCNT >> 1]) >= 3)) {
+				mLOG(GBA_MEM, GAME_ERROR, "Bad VRAM Load8: 0x%08X", address);
+				value = 0;
+				break;
+			}
 			value = ((uint8_t*) gba->video.vram)[address & 0x00017FFF];
+		} else {
+			value = ((uint8_t*) gba->video.vram)[address & 0x0001FFFF];
 		}
 		break;
 	case REGION_OAM:
@@ -717,20 +732,24 @@ uint32_t GBALoad8(struct ARMCore* cpu, uint32_t address, int* cycleCounter) {
 	wait += waitstatesRegion[REGION_PALETTE_RAM];
 
 #define STORE_VRAM \
-	if ((address & 0x0001FFFF) < SIZE_VRAM) { \
+	if ((address & 0x0001FFFF) >= SIZE_VRAM) { \
+		if ((address & (SIZE_VRAM | 0x00014000)) == SIZE_VRAM && (GBARegisterDISPCNTGetMode(gba->memory.io[REG_DISPCNT >> 1]) >= 3)) { \
+			mLOG(GBA_MEM, GAME_ERROR, "Bad VRAM Store32: 0x%08X", address); \
+		} else { \
+			LOAD_32(oldValue, address & 0x00017FFC, gba->video.vram); \
+			if (oldValue != value) { \
+				STORE_32(value, address & 0x00017FFC, gba->video.vram); \
+				gba->video.renderer->writeVRAM(gba->video.renderer, (address & 0x00017FFC) + 2); \
+				gba->video.renderer->writeVRAM(gba->video.renderer, (address & 0x00017FFC)); \
+			} \
+		} \
+	} else { \
 		LOAD_32(oldValue, address & 0x0001FFFC, gba->video.vram); \
 		if (oldValue != value) { \
 			STORE_32(value, address & 0x0001FFFC, gba->video.vram); \
 			gba->video.renderer->writeVRAM(gba->video.renderer, (address & 0x0001FFFC) + 2); \
 			gba->video.renderer->writeVRAM(gba->video.renderer, (address & 0x0001FFFC)); \
 		} \
-	} else { \
-		LOAD_32(oldValue, address & 0x00017FFC, gba->video.vram); \
-		if (oldValue != value) { \
-			STORE_32(value, address & 0x00017FFC, gba->video.vram); \
-			gba->video.renderer->writeVRAM(gba->video.renderer, (address & 0x00017FFC) + 2); \
-			gba->video.renderer->writeVRAM(gba->video.renderer, (address & 0x00017FFC)); \
-		} \
 	} \
 	wait += waitstatesRegion[REGION_VRAM];
 
@@ -753,12 +772,12 @@ uint32_t GBALoad8(struct ARMCore* cpu, uint32_t address, int* cycleCounter) {
 #define STORE_SRAM \
 	if (address & 0x3) { \
 		mLOG(GBA_MEM, GAME_ERROR, "Unaligned SRAM Store32: 0x%08X", address); \
-		value = 0; \
-	} \
-	GBAStore8(cpu, address & ~0x3, value, cycleCounter); \
-	GBAStore8(cpu, (address & ~0x3) | 1, value, cycleCounter); \
-	GBAStore8(cpu, (address & ~0x3) | 2, value, cycleCounter); \
-	GBAStore8(cpu, (address & ~0x3) | 3, value, cycleCounter);
+	} else { \
+		GBAStore8(cpu, address, value, cycleCounter); \
+		GBAStore8(cpu, address | 1, value, cycleCounter); \
+		GBAStore8(cpu, address | 2, value, cycleCounter); \
+		GBAStore8(cpu, address | 3, value, cycleCounter); \
+	}
 
 #define STORE_BAD \
 	mLOG(GBA_MEM, GAME_ERROR, "Bad memory Store32: 0x%08X", address);
@@ -840,18 +859,22 @@ void GBAStore16(struct ARMCore* cpu, uint32_t address, int16_t value, int* cycle
 		}
 		break;
 	case REGION_VRAM:
-		if ((address & 0x0001FFFF) < SIZE_VRAM) {
-			LOAD_16(oldValue, address & 0x0001FFFE, gba->video.vram);
-			if (value != oldValue) {
-				STORE_16(value, address & 0x0001FFFE, gba->video.vram);
-				gba->video.renderer->writeVRAM(gba->video.renderer, address & 0x0001FFFE);
+		if ((address & 0x0001FFFF) >= SIZE_VRAM) {
+			if ((address & (SIZE_VRAM | 0x00014000)) == SIZE_VRAM && (GBARegisterDISPCNTGetMode(gba->memory.io[REG_DISPCNT >> 1]) >= 3)) {
+				mLOG(GBA_MEM, GAME_ERROR, "Bad VRAM Store16: 0x%08X", address);
+				break;
 			}
-		} else {
 			LOAD_16(oldValue, address & 0x00017FFE, gba->video.vram);
 			if (value != oldValue) {
 				STORE_16(value, address & 0x00017FFE, gba->video.vram);
 				gba->video.renderer->writeVRAM(gba->video.renderer, address & 0x00017FFE);
 			}
+		} else {
+			LOAD_16(oldValue, address & 0x0001FFFE, gba->video.vram);
+			if (value != oldValue) {
+				STORE_16(value, address & 0x0001FFFE, gba->video.vram);
+				gba->video.renderer->writeVRAM(gba->video.renderer, address & 0x0001FFFE);
+			}
 		}
 		break;
 	case REGION_OAM:
@@ -892,7 +915,7 @@ void GBAStore16(struct ARMCore* cpu, uint32_t address, int16_t value, int* cycle
 			mLOG(GBA_MEM, INFO, "Detected EEPROM savegame");
 			GBASavedataInitEEPROM(&memory->savedata);
 		}
-		if (memory->savedata.type == SAVEDATA_EEPROM) {
+		if (memory->savedata.type == SAVEDATA_EEPROM512 || memory->savedata.type == SAVEDATA_EEPROM) {
 			GBASavedataWriteEEPROM(&memory->savedata, value, 1);
 			break;
 		}
@@ -900,8 +923,12 @@ void GBAStore16(struct ARMCore* cpu, uint32_t address, int16_t value, int* cycle
 		break;
 	case REGION_CART_SRAM:
 	case REGION_CART_SRAM_MIRROR:
-		GBAStore8(cpu, (address & ~0x1), value, cycleCounter);
-		GBAStore8(cpu, (address & ~0x1) | 1, value, cycleCounter);
+		if (address & 1) {
+			mLOG(GBA_MEM, GAME_ERROR, "Unaligned SRAM Store16: 0x%08X", address);
+			break;
+		}
+		GBAStore8(cpu, address, value, cycleCounter);
+		GBAStore8(cpu, address | 1, value, cycleCounter);
 		break;
 	default:
 		mLOG(GBA_MEM, GAME_ERROR, "Bad memory Store16: 0x%08X", address);
@@ -938,8 +965,7 @@ void GBAStore8(struct ARMCore* cpu, uint32_t address, int8_t value, int* cycleCo
 		GBAStore16(cpu, address & ~1, ((uint8_t) value) | ((uint8_t) value << 8), cycleCounter);
 		break;
 	case REGION_VRAM:
-		if ((address & 0x0001FFFF) >= ((GBARegisterDISPCNTGetMode(gba->memory.io[REG_DISPCNT >> 1]) == 4) ? 0x00014000 : 0x00010000)) {
-			// TODO: check BG mode
+		if ((address & 0x0001FFFF) >= ((GBARegisterDISPCNTGetMode(gba->memory.io[REG_DISPCNT >> 1]) >= 3) ? 0x00014000 : 0x00010000)) {
 			mLOG(GBA_MEM, GAME_ERROR, "Cannot Store8 to OBJ: 0x%08X", address);
 			break;
 		}
@@ -1304,6 +1330,12 @@ void GBAPatch8(struct ARMCore* cpu, uint32_t address, int8_t value, int8_t* old)
 }
 
 #define LDM_LOOP(LDM) \
+	if (UNLIKELY(!mask)) { \
+		LDM; \
+		cpu->gprs[ARM_PC] = value; \
+		wait += 16; \
+		address += 64; \
+	} \
 	for (i = 0; i < 16; i += 4) { \
 		if (UNLIKELY(mask & (1 << i))) { \
 			LDM; \
@@ -1416,6 +1448,12 @@ uint32_t GBALoadMultiple(struct ARMCore* cpu, uint32_t address, int mask, enum L
 }
 
 #define STM_LOOP(STM) \
+	if (UNLIKELY(!mask)) { \
+		value = cpu->gprs[ARM_PC] + (cpu->executionMode == MODE_ARM ? WORD_SIZE_ARM : WORD_SIZE_THUMB); \
+		STM; \
+		wait += 16; \
+		address += 64; \
+	} \
 	for (i = 0; i < 16; i += 4) { \
 		if (UNLIKELY(mask & (1 << i))) { \
 			value = cpu->gprs[i]; \
@@ -1647,6 +1685,10 @@ void _pristineCow(struct GBA* gba) {
 }
 
 void GBAPrintFlush(struct GBA* gba) {
+	if (!gba->memory.agbPrintBuffer) {
+		return;
+	}
+
 	char oolBuf[0x101];
 	size_t i;
 	for (i = 0; gba->memory.agbPrintCtx.get != gba->memory.agbPrintCtx.put && i < 0x100; ++i) {
@@ -1688,8 +1730,8 @@ static void _agbPrintStore(struct GBA* gba, uint32_t address, int16_t value) {
 
 static int16_t _agbPrintLoad(struct GBA* gba, uint32_t address) {
 	struct GBAMemory* memory = &gba->memory;
-	int16_t value = 0xFFFF;
-	if (address < AGB_PRINT_TOP) {
+	int16_t value = address >> 1;
+	if (address < AGB_PRINT_TOP && memory->agbPrintBuffer) {
 		LOAD_16(value, address & (SIZE_AGB_PRINT - 1), memory->agbPrintBuffer);
 	} else if ((address & 0x00FFFFF8) == (AGB_PRINT_STRUCT & 0x00FFFFF8)) {
 		value = (&memory->agbPrintCtx.request)[(address & 7) >> 1];
