@@ -13,19 +13,21 @@
 #include "gba/cheats/parv3.h"
 #include "gba/cheats/gameshark.h"
 
-M_TEST_SUITE_SETUP(GBACheats) {
+static int cheatsSetup(void** state) {
 	struct mCore* core = GBACoreCreate();
 	core->init(core);
+	mCoreInitConfig(core, NULL);
 	core->cheatDevice(core);
 	*state = core;
 	return 0;
 }
 
-M_TEST_SUITE_TEARDOWN(GBACheats) {
+static int cheatsTeardown(void** state) {
 	if (!*state) {
 		return 0;
 	}
 	struct mCore* core = *state;
+	mCoreConfigDeinit(&core->config);
 	core->deinit(core);
 	return 0;
 }
@@ -36,7 +38,7 @@ M_TEST_DEFINE(createSet) {
 	assert_non_null(device);
 	struct mCheatSet* set = device->createSet(device, NULL);
 	assert_non_null(set);
-	set->deinit(set);
+	mCheatSetDeinit(set);
 }
 
 M_TEST_DEFINE(addRawPARv3) {
@@ -48,7 +50,7 @@ M_TEST_DEFINE(addRawPARv3) {
 	GBACheatSetGameSharkVersion((struct GBACheatSet*) set, GBA_GS_PARV3_RAW);
 	assert_true(set->addLine(set, "80000000 00000000", GBA_CHEAT_PRO_ACTION_REPLAY));
 	assert_false(set->addLine(set, "43000000 00000000", GBA_CHEAT_PRO_ACTION_REPLAY));
-	set->deinit(set);
+	mCheatSetDeinit(set);
 }
 
 M_TEST_DEFINE(doPARv3Assign) {
@@ -72,7 +74,7 @@ M_TEST_DEFINE(doPARv3Assign) {
 	assert_int_equal(core->rawRead16(core, 0x03000002, -1), 0x5678);
 	assert_int_equal(core->rawRead32(core, 0x03000004, -1), 0x12345678);
 
-	set->deinit(set);
+	mCheatSetDeinit(set);
 }
 
 M_TEST_DEFINE(doPARv3Slide1) {
@@ -101,7 +103,7 @@ M_TEST_DEFINE(doPARv3Slide1) {
 	assert_int_equal(core->rawRead8(core, 0x03000004, -1), 0);
 	assert_int_equal(core->rawRead8(core, 0x03000005, -1), 0);
 
-	set->deinit(set);
+	mCheatSetDeinit(set);
 }
 
 M_TEST_DEFINE(doPARv3Slide2) {
@@ -130,7 +132,7 @@ M_TEST_DEFINE(doPARv3Slide2) {
 	assert_int_equal(core->rawRead16(core, 0x03000008, -1), 0);
 	assert_int_equal(core->rawRead16(core, 0x0300000A, -1), 0);
 
-	set->deinit(set);
+	mCheatSetDeinit(set);
 }
 
 M_TEST_DEFINE(doPARv3Slide4) {
@@ -159,7 +161,7 @@ M_TEST_DEFINE(doPARv3Slide4) {
 	assert_int_equal(core->rawRead16(core, 0x03000010, -1), 0);
 	assert_int_equal(core->rawRead16(core, 0x03000014, -1), 0);
 
-	set->deinit(set);
+	mCheatSetDeinit(set);
 }
 
 M_TEST_DEFINE(doPARv3If1) {
@@ -188,7 +190,7 @@ M_TEST_DEFINE(doPARv3If1) {
 	assert_int_equal(core->rawRead8(core, 0x03000000, -1), 0x1);
 	assert_int_equal(core->rawRead8(core, 0x03000001, -1), 0x11);
 
-	set->deinit(set);
+	mCheatSetDeinit(set);
 }
 
 M_TEST_DEFINE(doPARv3If1x1) {
@@ -243,7 +245,7 @@ M_TEST_DEFINE(doPARv3If1x1) {
 	assert_int_equal(core->rawRead8(core, 0x03000002, -1), 0x21);
 	assert_int_equal(core->rawRead8(core, 0x03000003, -1), 0x31);
 
-	set->deinit(set);
+	mCheatSetDeinit(set);
 }
 
 M_TEST_DEFINE(doPARv3If2) {
@@ -277,7 +279,7 @@ M_TEST_DEFINE(doPARv3If2) {
 	assert_int_equal(core->rawRead8(core, 0x03000001, -1), 0x11);
 	assert_int_equal(core->rawRead8(core, 0x03000002, -1), 0x21);
 
-	set->deinit(set);
+	mCheatSetDeinit(set);
 }
 
 M_TEST_DEFINE(doPARv3If2x2) {
@@ -345,7 +347,7 @@ M_TEST_DEFINE(doPARv3If2x2) {
 	assert_int_equal(core->rawRead8(core, 0x03000003, -1), 0x31);
 	assert_int_equal(core->rawRead8(core, 0x03000004, -1), 0x41);
 	assert_int_equal(core->rawRead8(core, 0x03000005, -1), 0x51);
-	set->deinit(set);
+	mCheatSetDeinit(set);
 }
 
 M_TEST_DEFINE(doPARv3If2Contain1) {
@@ -392,7 +394,7 @@ M_TEST_DEFINE(doPARv3If2Contain1) {
 	assert_int_equal(core->rawRead8(core, 0x03000000, -1), 0x1);
 	assert_int_equal(core->rawRead8(core, 0x03000001, -1), 0x1);
 	assert_int_equal(core->rawRead8(core, 0x03000002, -1), 0x21);
-	set->deinit(set);
+	mCheatSetDeinit(set);
 }
 
 M_TEST_DEFINE(doPARv3IfX) {
@@ -421,7 +423,7 @@ M_TEST_DEFINE(doPARv3IfX) {
 	mCheatRefresh(device, set);
 	assert_int_equal(core->rawRead8(core, 0x03000000, -1), 0x1);
 	assert_int_equal(core->rawRead8(core, 0x03000001, -1), 0x11);
-	set->deinit(set);
+	mCheatSetDeinit(set);
 }
 
 M_TEST_DEFINE(doPARv3IfXxX) {
@@ -484,7 +486,7 @@ M_TEST_DEFINE(doPARv3IfXxX) {
 	assert_int_equal(core->rawRead8(core, 0x03000002, -1), 0x21);
 	assert_int_equal(core->rawRead8(core, 0x03000003, -1), 0x32);
 	assert_int_equal(core->rawRead8(core, 0x03000004, -1), 0x41);
-	set->deinit(set);
+	mCheatSetDeinit(set);
 }
 
 M_TEST_DEFINE(doPARv3IfXElse) {
@@ -519,7 +521,7 @@ M_TEST_DEFINE(doPARv3IfXElse) {
 	assert_int_equal(core->rawRead8(core, 0x03000000, -1), 0x1);
 	assert_int_equal(core->rawRead8(core, 0x03000001, -1), 0x11);
 	assert_int_equal(core->rawRead8(core, 0x03000002, -1), 0x22);
-	set->deinit(set);
+	mCheatSetDeinit(set);
 }
 
 M_TEST_DEFINE(doPARv3IfXElsexX) {
@@ -590,7 +592,7 @@ M_TEST_DEFINE(doPARv3IfXElsexX) {
 	assert_int_equal(core->rawRead8(core, 0x03000003, -1), 0x32);
 	assert_int_equal(core->rawRead8(core, 0x03000004, -1), 0x42);
 	assert_int_equal(core->rawRead8(core, 0x03000005, -1), 0x51);
-	set->deinit(set);
+	mCheatSetDeinit(set);
 }
 
 M_TEST_DEFINE(doPARv3IfXElsexXElse) {
@@ -668,7 +670,7 @@ M_TEST_DEFINE(doPARv3IfXElsexXElse) {
 	assert_int_equal(core->rawRead8(core, 0x03000004, -1), 0x42);
 	assert_int_equal(core->rawRead8(core, 0x03000005, -1), 0x51);
 	assert_int_equal(core->rawRead8(core, 0x03000006, -1), 0x62);
-	set->deinit(set);
+	mCheatSetDeinit(set);
 }
 
 M_TEST_DEFINE(doPARv3IfXContain1) {
@@ -730,7 +732,7 @@ M_TEST_DEFINE(doPARv3IfXContain1) {
 	assert_int_equal(core->rawRead8(core, 0x03000002, -1), 0x21);
 	assert_int_equal(core->rawRead8(core, 0x03000003, -1), 0x31);
 	assert_int_equal(core->rawRead8(core, 0x03000004, -1), 0x41);
-	set->deinit(set);
+	mCheatSetDeinit(set);
 }
 
 M_TEST_DEFINE(doPARv3IfXContain1Else) {
@@ -800,7 +802,7 @@ M_TEST_DEFINE(doPARv3IfXContain1Else) {
 	assert_int_equal(core->rawRead8(core, 0x03000003, -1), 0x31);
 	assert_int_equal(core->rawRead8(core, 0x03000004, -1), 0x41);
 	assert_int_equal(core->rawRead8(core, 0x03000005, -1), 0x52);
-	set->deinit(set);
+	mCheatSetDeinit(set);
 }
 
 M_TEST_DEFINE(doPARv3IfXElseContain1) {
@@ -870,7 +872,7 @@ M_TEST_DEFINE(doPARv3IfXElseContain1) {
 	assert_int_equal(core->rawRead8(core, 0x03000003, -1), 0x32);
 	assert_int_equal(core->rawRead8(core, 0x03000004, -1), 0x41);
 	assert_int_equal(core->rawRead8(core, 0x03000005, -1), 0x52);
-	set->deinit(set);
+	mCheatSetDeinit(set);
 }
 
 M_TEST_DEFINE(doPARv3IfXContain1ElseContain1) {
@@ -1016,7 +1018,7 @@ M_TEST_DEFINE(doPARv3IfXContain1ElseContain1) {
 	assert_int_equal(core->rawRead8(core, 0x03000006, -1), 0x62);
 	assert_int_equal(core->rawRead8(core, 0x03000007, -1), 0x71);
 	assert_int_equal(core->rawRead8(core, 0x03000008, -1), 0x82);
-	set->deinit(set);
+	mCheatSetDeinit(set);
 }
 
 M_TEST_DEFINE(doPARv3IfButton) {
@@ -1046,28 +1048,28 @@ M_TEST_DEFINE(doPARv3IfButton) {
 	core->rawWrite8(core, 0x03000000, -1, 0);
 	mCheatRefresh(device, set);
 	assert_int_equal(core->rawRead8(core, 0x03000000, -1), 0);
-	set->deinit(set);
+	mCheatSetDeinit(set);
 }
 
-M_TEST_SUITE_DEFINE_SETUP_TEARDOWN(GBACheats,
-	cmocka_unit_test(createSet),
-	cmocka_unit_test(addRawPARv3),
-	cmocka_unit_test(doPARv3Assign),
-	cmocka_unit_test(doPARv3Slide1),
-	cmocka_unit_test(doPARv3Slide2),
-	cmocka_unit_test(doPARv3Slide4),
-	cmocka_unit_test(doPARv3If1),
-	cmocka_unit_test(doPARv3If1x1),
-	cmocka_unit_test(doPARv3If2),
-	cmocka_unit_test(doPARv3If2x2),
-	cmocka_unit_test(doPARv3If2Contain1),
-	cmocka_unit_test(doPARv3IfX),
-	cmocka_unit_test(doPARv3IfXxX),
-	cmocka_unit_test(doPARv3IfXElse),
-	cmocka_unit_test(doPARv3IfXElsexX),
-	cmocka_unit_test(doPARv3IfXElsexXElse),
-	cmocka_unit_test(doPARv3IfXContain1),
-	cmocka_unit_test(doPARv3IfXContain1Else),
-	cmocka_unit_test(doPARv3IfXElseContain1),
-	cmocka_unit_test(doPARv3IfXContain1ElseContain1),
-	cmocka_unit_test(doPARv3IfButton))
+M_TEST_SUITE_DEFINE(GBACheats,
+	cmocka_unit_test_setup_teardown(createSet, cheatsSetup, cheatsTeardown),
+	cmocka_unit_test_setup_teardown(addRawPARv3, cheatsSetup, cheatsTeardown),
+	cmocka_unit_test_setup_teardown(doPARv3Assign, cheatsSetup, cheatsTeardown),
+	cmocka_unit_test_setup_teardown(doPARv3Slide1, cheatsSetup, cheatsTeardown),
+	cmocka_unit_test_setup_teardown(doPARv3Slide2, cheatsSetup, cheatsTeardown),
+	cmocka_unit_test_setup_teardown(doPARv3Slide4, cheatsSetup, cheatsTeardown),
+	cmocka_unit_test_setup_teardown(doPARv3If1, cheatsSetup, cheatsTeardown),
+	cmocka_unit_test_setup_teardown(doPARv3If1x1, cheatsSetup, cheatsTeardown),
+	cmocka_unit_test_setup_teardown(doPARv3If2, cheatsSetup, cheatsTeardown),
+	cmocka_unit_test_setup_teardown(doPARv3If2x2, cheatsSetup, cheatsTeardown),
+	cmocka_unit_test_setup_teardown(doPARv3If2Contain1, cheatsSetup, cheatsTeardown),
+	cmocka_unit_test_setup_teardown(doPARv3IfX, cheatsSetup, cheatsTeardown),
+	cmocka_unit_test_setup_teardown(doPARv3IfXxX, cheatsSetup, cheatsTeardown),
+	cmocka_unit_test_setup_teardown(doPARv3IfXElse, cheatsSetup, cheatsTeardown),
+	cmocka_unit_test_setup_teardown(doPARv3IfXElsexX, cheatsSetup, cheatsTeardown),
+	cmocka_unit_test_setup_teardown(doPARv3IfXElsexXElse, cheatsSetup, cheatsTeardown),
+	cmocka_unit_test_setup_teardown(doPARv3IfXContain1, cheatsSetup, cheatsTeardown),
+	cmocka_unit_test_setup_teardown(doPARv3IfXContain1Else, cheatsSetup, cheatsTeardown),
+	cmocka_unit_test_setup_teardown(doPARv3IfXElseContain1, cheatsSetup, cheatsTeardown),
+	cmocka_unit_test_setup_teardown(doPARv3IfXContain1ElseContain1, cheatsSetup, cheatsTeardown),
+	cmocka_unit_test_setup_teardown(doPARv3IfButton, cheatsSetup, cheatsTeardown))
