@@ -1024,7 +1024,7 @@ const QList<IOViewer::RegisterDescription>& IOViewer::registerDescriptions() {
 }
 
 IOViewer::IOViewer(std::shared_ptr<CoreController> controller, QWidget* parent)
-	: QDialog(parent)
+	: QDialog(parent, Qt::WindowTitleHint | Qt::WindowSystemMenuHint | Qt::WindowCloseButtonHint)
 	, m_controller(controller)
 {
 	m_ui.setupUi(this);
